import json
from playwright.async_api import async_playwright, ElementHandle
from .config import Config
config = Config()
async def create_bot_async(page, botname, base_bot_index, prompt, retries=2):
    try:
        for i in range(retries):
            await page.goto('https://poe.com/create_bot')

            # 定位输入框并清空原有的默认值
            name_input: ElementHandle = await page.wait_for_selector('input[name="name"]')
            await name_input.fill('')

            # 输入新的值
            await name_input.fill(botname)

            await page.wait_for_selector('select[name="baseBot"]')
            
            if base_bot_index == 1:
                value = "chinchilla"
            elif base_bot_index ==2:
                value = "a2"
            # 根据索引选择选项
            await page.select_option('.BotInfoForm_select__lFCl0', value=value)
            
            # 添加预设
            prompt_textarea = await page.wait_for_selector('textarea[name="prompt"]')
            await prompt_textarea.fill(prompt)

            chevron_button = await page.wait_for_selector('.BotInfoForm_chevronDown__LFWWC')
            await chevron_button.click()

            # 点击"Suggest replies"复选框的<label>元素
            checkbox_label: ElementHandle = await page.wait_for_selector('//div[contains(text(), "Suggest replies")]/following-sibling::label')
            await checkbox_label.click()

            # 点击"Linkify bot responses"复选框的<label>元素
            checkbox_label = await page.wait_for_selector('//div[contains(text(), "Linkify bot responses")]/following-sibling::label')
            await checkbox_label.click()

            # 点击"Create bot"按钮
            create_bot_button = await page.wait_for_selector('button.Button_primary__pIDjn')
            await create_bot_button.click()

            # 等待新页面加载完成
            try:
                await page.wait_for_selector('textarea.ChatMessageInputView_textInput__Aervw', timeout=5000)
                return True
            except:
                pass

        return False
    finally:
        await page.close()
# cookie_path = r"C:\Users\Administrator\Desktop\nonebot2\cccc\src\plugins\nonebot_poe_chat\data\poe_cookie.json"                      
async def poe_create(cookie_path,botname, base_bot_index, prompt):
    async with async_playwright() as p:
        server = config.server
        username = config.username
        passwd = config.passwd
        proxy_config = {}
        if server is not None:
            proxy_config["server"] = server
        if username is not None:
            proxy_config["username"] = username
        if passwd is not None:
            proxy_config["password"] = passwd

        if proxy_config:
            browser = await p.chromium.launch(proxy=proxy_config)
        else:
            browser = await p.chromium.launch()
        context = await browser.new_context()
        page = await context.new_page()
        with open(cookie_path, 'r') as f:
            cookies = json.load(f)
        await context.add_cookies(cookies=cookies)
        is_created = await create_bot_async(page,botname, base_bot_index, prompt, retries=5)
        await context.close()
        await browser.close()
        return is_created




import json,asyncio
from nonebot.plugin import on_command, on
from nonebot.params import ArgStr, CommandArg
from nonebot.typing import T_State
from nonebot.matcher import Matcher
from nonebot.adapters.onebot.v11 import Message, Event, Bot, MessageEvent,MessageSegment
from nonebot.internal.rule import Rule
from playwright.async_api import async_playwright
from .poe_chat import poe_chat
from .poe_create import poe_create
from .poe_clear import poe_clear
from .poe_login import submit_email, submit_code
from .config import Config
from .poe_func import reply_out,generate_uuid,generate_random_string,is_email
from .txt2img import Txt2Img
txt2img = Txt2Img()

#一些配置
config = Config()
server = config.server
username = config.username
passwd = config.passwd
proxy_config = {}
if server is not None:
    proxy_config["server"] = server
if username is not None:
    proxy_config["username"] = username
if passwd is not None:
    proxy_config["password"] = passwd
user_dict = config.user_dict
prompts_dict = config.prompts_dict
user_path = config.user_path
prompt_path = config.prompt_path
cookie_path = config.cookie_path
superusers = config.superusers
is_cookie_exists = config.is_cookie_exists
is_pic_able = config.pic_able
######################################################

async def delete_messages(bot, user_id, dict_list):
    if user_id in dict_list:
        for eachmsg in dict_list[user_id]:
            await bot.delete_msg(message_id=eachmsg['message_id'])
        del dict_list[user_id]
create_msgs = {}
poe_create_ = on_command(
    "poecreate",
    aliases={
        "创建bot",
        "pc"
        },
    priority=4,
    block=False)
@poe_create_.handle()
async def __(matcher:Matcher,state: T_State,event: Event):
    create_msgs[str(event.user_id)] = []
    state["user_id"] = str(event.user_id)
    if not is_cookie_exists:
        await matcher.finish(reply_out(event, "管理员还没填写可用的poe_cookie或登陆"))
    else:
        if len(prompts_dict)>0:
            str_prompts = str()
            for key, _ in prompts_dict.items():
                str_prompts += f"{key}\n"
            # create_msgs[str(event.user_id)].append(await matcher.send(reply_out(event, f"当前预设有：\n{str_prompts}")))
            msg = f"当前预设有：\n{str_prompts}\n请输入\n1.机器人名称,\n2.基础模型选项，可选项为（gpt3_5输入1,claude输入2）,\n3.自定义预设（预设内容中间不要有空格） 或 \".\" + 可用本地预设名\n三个参数中间用空格隔开\n最终格式示例:\n示例一：chat 2 一个智能助理\n示例二： claude 1 .默认\n输入取消 或 算了可以终止创建"
            create_msgs[str(event.user_id)].append(await matcher.send(reply_out(event, msg)))
        else:
            create_msgs[str(event.user_id)].append(await matcher.send(reply_out(event, "")))
            msg = f"当前没有可用本地预设\n\n请输入\n1.机器人名称,\n2.基础模型选项，可选项为（gpt3_5输入1,claude输入2）,\n3.自定义预设（预设内容中间不要有空格） 或 \".\" + 可用本地预设名\n三个参数中间用空格隔开\n最终格式示例:\n示例一：chat 2 一个智能助理\n示例二： claude 1 .默认\n输入取消 或 算了可以终止创建"
            create_msgs[str(event.user_id)].append(await matcher.send(reply_out(event, msg)))

@poe_create_.got('model')
async def __poe_create___(bot: Bot,matcher: Matcher,event: Event,state: T_State, infos: str = ArgStr("model")):
    if infos in ["取消", "算了"]:
        create_msgs[str(event.user_id)].append(await matcher.send(reply_out(event, "取消创建")))
        await asyncio.sleep(1)
        await delete_messages(bot,str(event.user_id),create_msgs)
        await poe_create_.finish()
    infos = infos.split(" ")
    if not (len(infos) == 3 and infos[1] in ['1', '2']):
        create_msgs[str(event.user_id)].append(await matcher.send(reply_out(event, "输入信息有误，请检查后重新输入")))
        await poe_create_.reject()
    #获取创建所需信息
    userid = str(state['user_id'])
    nickname = str(infos[0])
    truename = str(generate_uuid(str(userid + infos[0])))
    prompt = str(infos[2])
    bot_index = str(infos[1])
    if prompt.startswith("."):
        prompt_name = prompt[1:]
        if prompt_name in prompts_dict:
            prompt = prompts_dict[prompt_name]
        else:
            create_msgs[str(event.user_id)].append(await matcher.send(reply_out(event, "输入的本地预设名不正确，请重新输入")))
            await poe_create_.reject()
    
    if not userid in user_dict:
        user_dict[userid] = {}
    if 'all' not in user_dict[userid]:
        user_dict[userid]['all'] = {}
    if 'now' not in user_dict[userid]:
        user_dict[userid]['now'] = {}
    #查看对应用户下是不是有重名的bot
    if  nickname in user_dict[userid]['all']:
        create_msgs[str(event.user_id)].append(await matcher.send(reply_out(event, "已经有同名的bot了，换一个名字重新输入吧")))
        await poe_create_.reject()
    else:
        is_created = await poe_create(cookie_path,truename,int(bot_index),prompt)
        if is_created:
            # # 将更新后的字典写回到JSON文件中
            user_dict[userid]['all'][nickname] = truename
            
            if user_dict[userid]['now']:
                user_dict[userid]['now'] = {}
            user_dict[userid]['now'][nickname] = truename
            
            with open(user_path, 'w') as f:
                json.dump(user_dict, f)

            await matcher.send(reply_out(event, "创建成功并切换到新建bot"))
            await asyncio.sleep(1)
            await delete_messages(bot,userid,create_msgs)
            await poe_switch.finish()
        else: 
            await matcher.send(reply_out(event, "出错了，多次出错请联系机器人管理员"))
            await asyncio.sleep(1)
            await delete_messages(bot,userid,create_msgs)
            await poe_switch.finish()

######################################################    

#保留上一个回答的chatsuggest
chat_lock = asyncio.Semaphore(3)
chat_suggest = {}
last_messageid = {}
waitque = []
poe_chat_ = on_command(
    "poetalk",
    aliases={
        "ptalk",
        "pt"
        },
    priority=1,
    block=False)
@poe_chat_.handle()
async def __chat_bot__(matcher:Matcher,event: Event, args: Message = CommandArg()):
    global chat_lock,chat_suggest,last_messageid
    userid = str(event.user_id)
    if not is_cookie_exists:
        await matcher.finish(reply_out(event, "管理员还没填写可用的poe_cookie或登陆"))
    if userid not in user_dict:
        random = generate_random_string()
        truename = str(generate_uuid(str(userid + random)))
        is_created = await poe_create(cookie_path,truename,1,"一个智能助理")
        if is_created:
            user_dict[userid] = {}        
            user_dict[userid]['all'] = {}
            user_dict[userid]['all']["default"] = truename
            user_dict[userid]['now'] = {}
            user_dict[userid]['now']["default"] = truename
            with open(user_path, 'w') as f:
                json.dump(user_dict, f)
            await matcher.send(reply_out(event, "自动创建gpt3.5成功"))
        else:
            await matcher.send(reply_out(event, "自动创建gpt3.5失败，多次失败请联系机器人管理员"))
    if userid in chat_suggest and len(str(args[0])) == 1 and str(args[0]) in ['1','2','3','4']:
        text = chat_suggest[userid][int(str(args[0]))-1]
    else:
        text = str(args[0])
    botname = str(list(user_dict[userid]["now"].values())[0])
    if userid in waitque:
        await matcher.finish(reply_out(event, "你已经有一个对话进行中了，请等结束后再发送"))
    if chat_lock.locked():
        await matcher.send(reply_out(event, "请稍等,你前面已有3个用户,你的回答稍后就来"))
    async with chat_lock:
        waitque.append(userid)
        result = await poe_chat(cookie_path, botname, text)
        if isinstance(result, tuple):
            last_answer, chat_suggest[userid] = result
            is_successful = True
        elif isinstance(result, str):
            if "banned" == result:
                waitque.remove(userid)
                await matcher.finish(reply_out(event, '你的机器人被banned了，请/pc新建一个机器人，并且不要在使用此预设'))
        elif isinstance(result, bool):
            is_successful = result
        else:
            #这个应该不可能出现
            raise ValueError("Unexpected return type from get_message_async")

        if is_successful:
            suggest_str = '\n'.join([f"{i+1}: {s}" for i, s in enumerate(chat_suggest[userid])])
            msg = f"{last_answer}\n\n建议回复：\n{suggest_str}"
            waitque.remove(userid)
            if is_pic_able:
                pic = await txt2img.draw(title=" ",text=msg)
                last_messageid[userid] = await matcher.send(reply_out(event, pic))
                matcher.finish()
            else:
                await matcher.finish(reply_out(event, msg))
        else:
            waitque.remove(userid)
            await matcher.finish(reply_out(event, "出错了，多次出错请联系机器人管理员"))

######################################################
#判断是不是同一个对话中
async def _is_reply_(event:MessageEvent,bot:Bot):
    if bool(event.reply):
        reply = event.reply
        user_id = str(event.user_id)
        last_message_id = last_messageid[user_id]["message_id"]
        bot_id = bot.self_id
        sender_id = str(reply.sender.user_id)
        lastsuggest = str(reply.message).split("\n")[-1][3:]
        if user_id in chat_suggest and sender_id == bot_id and (lastsuggest == chat_suggest[user_id][-1] or last_message_id==reply.message_id):
            return True
    return False
is_reply = Rule(_is_reply_)

_poe_continue_ = on(rule=is_reply)
@_poe_continue_.handle()
async def __poe_continue__(matcher: Matcher,event:MessageEvent):
    userid = str(event.user_id)
    raw_message = str(event.message)
    if userid in waitque:
        await matcher.finish(reply_out(event, "你已经有一个对话进行中了，请等结束后在发送"))
    if userid in chat_suggest and len(raw_message) == 1 and raw_message in ['1','2','3','4']:
            text = chat_suggest[userid][int(raw_message)-1]
    else:
        text = raw_message
    
    global chat_lock,last_messageid
    if chat_lock.locked():
        await matcher.send(reply_out(event, '请稍等,你前面已有3个用户'))
        
    botname = str(list(user_dict[userid]["now"].values())[0])
        
    async with chat_lock:
        waitque.append(userid)
        result = await poe_chat(cookie_path, botname, text)
        if isinstance(result, tuple):
            last_answer, chat_suggest[userid] = result
            is_successful = True
        elif isinstance(result, str):
            if "banned" == result:
                waitque.remove(userid)
                await matcher.finish(reply_out(event, '你的机器人被banned了，请/pc新建一个机器人，并且不要在使用此预设'))
        elif isinstance(result, bool):
            is_successful = result
        else:
            raise ValueError("Unexpected return type from get_message_async")

        if is_successful:
            suggest_str = '\n'.join([f"{i+1}: {s}" for i, s in enumerate(chat_suggest[userid])])
            msg = f"{last_answer}\n\n建议回复：\n{suggest_str}"
            waitque.remove(userid)
            if is_pic_able:
                pic = await txt2img.draw(title=" ",text=msg)
                last_messageid[userid] = await matcher.send(reply_out(event, pic))
                matcher.finish()
            else:
                await matcher.finish(reply_out(event, msg))
        else:
            waitque.remove(userid)
            await matcher.finish(reply_out(event, "出错了，多次出错请联系机器人管理员"))
    
######################################################   
poe_clear_ = on_command(
    "poedump",
    aliases={
        "poe清除",
        "pd"
        },
    priority=4,
    block=False)
@poe_clear_.handle()
async def __poe_clear___(event: Event,matcher:Matcher):
    if not is_cookie_exists:
        await matcher.finish(reply_out(event, "机器人管理员还没填写可用的poe_cookie或登陆"))
    userid = str(event.user_id)
    
    if userid not in user_dict:
        await matcher.finish(reply_out(event, "你还没有创建任何bot"))
    if userid in waitque:
        await matcher.finish(reply_out(event, "你现在有一个回话在进行中，不能清除历史记录"))
    botname = str(list(user_dict[userid]["now"].values())[0])
    nickname = str(list(user_dict[userid]["now"].keys())[0])
    is_cleared = await poe_clear(cookie_path,botname)
    if is_cleared:
        msg = f"成功清除了{nickname}的历史消息"
    else:
        msg = "出错了，多次错误请联系机器人主人"
    await matcher.finish(reply_out(event, msg))
switch_msgs = {}    
######################################################
poe_switch = on_command(
    "poeswitch",
    aliases={
        "切换bot",
        "ps"
        },
    priority=4,
    block=False)
@poe_switch.handle()
async def __poe_switch__(bot:Bot ,matcher:Matcher,event: Event):
    if not is_cookie_exists:
        await matcher.finish(reply_out(event, "管理员还没填写可用的poe_cookie或登陆"))
    userid = str(event.user_id)
    switch_msgs[userid] = []
    if userid not in user_dict:
        await matcher.finish(reply_out(event, "你还没创建任何bot"))
    bots = list(user_dict[userid]["all"].keys())
    bot_str = '\n'.join(str(bot) for bot in bots)
    # bot_truname = str(list(user_dict[userid]["now"].values())[0])
    nickname = str(list(user_dict[userid]["now"].keys())[0])
    if len(bots)==1:
        switch_msgs[userid].append(await matcher.send(reply_out(event, f"当前只有一个bot:{nickname}")))
        await asyncio.sleep(1)
        await delete_messages(bot,userid,switch_msgs)
        await poe_switch.finish()
    msg = "你已经创建的的bot有：\n" + bot_str +f"\n当前使用的bot是{nickname}\n\n请输入要切换的机器人名称"
    switch_msgs[userid].append(await matcher.send(reply_out(event, msg)))

@poe_switch.got('nickname')
async def __poe_switch____(bot:Bot,matcher:Matcher,event: Event, infos: str = ArgStr("nickname")):
    userid = str(event.user_id)
    bots = list(user_dict[userid]["all"].keys())
    if infos in ["取消", "算了"]:
        switch_msgs[userid].append(await matcher.send(reply_out(event, "中断切换")))
        await asyncio.sleep(1)
        await delete_messages(bot,userid,switch_msgs)
        await poe_switch.finish()
    infos = infos.split(" ")
    nickname = infos[0]
    if not (nickname in bots):
        switch_msgs[userid].append(await matcher.send(reply_out(event, "输入信息有误，请检查后重新输入")))
        await poe_switch.reject()
    to_truename = user_dict[userid]["all"][nickname]
    del user_dict[userid]["now"]
    user_dict[userid]["now"] = {}
    user_dict[userid]["now"][nickname] = to_truename
    with open(user_path, 'w') as f:
        json.dump(user_dict, f)
    msg = f"已切换为{nickname}"
    await matcher.send(reply_out(event, msg))
    await asyncio.sleep(1)
    await delete_messages(bot,userid,switch_msgs)
    await poe_switch.finish()
remove_list = {}    
######################################################
poe_remove = on_command(
    "poeremove",
    aliases={
        "删除bot",
        "pr"
        },
    priority=4,
    block=False)
@poe_remove.handle()
async def __poe_remove__(matcher:Matcher,event: Event):
    if not is_cookie_exists:
        await matcher.finish(reply_out(event, "管理员还没填写可用的poe_cookie或登陆"))
    userid = str(event.user_id)
    remove_list[userid] = []
    if userid not in user_dict:
        await matcher.finish(reply_out(event, "你还没创建任何bot"))
    bots = list(user_dict[userid]["all"].keys())
    bot_str = '\n'.join(str(bot) for bot in bots)
    # bot_truname = str(list(user_dict[userid]["now"].values())[0])
    nickname = str(list(user_dict[userid]["now"].keys())[0])
    msg = "你已经创建的的bot有：\n" + bot_str +f"\n当前使用的bot是{nickname}\n\n请输入要删除的机器人名称"
    remove_list.append(await matcher.send(reply_out(event, msg)))

@poe_remove.got('nickname')
async def __poe_remove____(bot:Bot,matcher:Matcher,event: Event, infos: str = ArgStr("nickname")):
    userid = str(event.user_id)
    bots = list(user_dict[userid]["all"].keys())
    if infos in ["取消", "算了"]:
        remove_list.append(await matcher.send(reply_out(event, "终止切换")))
        await asyncio.sleep(1)
        await delete_messages(bot,userid,remove_list)
    infos = infos.split(" ")
    nickname_delete = infos[0]
    nickname_now = str(list(user_dict[userid]["now"].keys())[0])
    if not (nickname_delete in bots): 
        remove_list.append(await matcher.send(reply_out(event, "输入信息有误，请检查后重新输入")))
        await poe_remove.reject()
    if nickname_delete == nickname_now:
        remove_list.append(await matcher.send(reply_out(event, "不能删除正在使用的bot哦")))
        await asyncio.sleep(1)
        await delete_messages(bot,userid,remove_list)
        await poe_remove.finish()
    del user_dict[userid]["all"][nickname_delete]
    with open(user_path, 'w') as f:
        json.dump(user_dict, f)
    await matcher.finish(reply_out(event, f"已删除{nickname_delete}"))
    
#####################################################

poe_login = on_command(
    "poelogin",
    aliases={
        "poe登陆",
        "pl"
        },
    priority=4,
    block=False)

global_playwright = None
#创建一个全局的浏览器，在两个def中使用
async def create_playwright_instance():
    global global_playwright
    if not global_playwright:
        global_playwright = await async_playwright().start()
    return global_playwright

@poe_login.got('mail', prompt='请输入邮箱')
async def __poe_login____(event: Event, state: T_State, infos: str = ArgStr("mail")):
    global driver, global_playwright
    userid = str(event.user_id)
    if infos in ["取消", "算了"]:
        await poe_login.finish("终止登陆")
    infos = infos.split(" ")
    if len(infos) != 1 or not is_email(infos[0]):
        await poe_login.reject("你输入的邮箱有误，请重新输入")

    playwright = await create_playwright_instance()
    if proxy_config:
        browser = await playwright.chromium.launch(proxy=proxy_config,headless=False)
    else:
        browser = await playwright.chromium.launch()
    context = await browser.new_context()
    page = await context.new_page()
    state["playwright"] = playwright
    state["browser"] = browser
    state["context"] = context
    state["page"] = page

    is_submitted_email = await submit_email(page, infos[0])
    if is_submitted_email:
        await poe_login.send("填写邮箱成功")
    else:
        await poe_login.finish("出错了，可以换个邮箱试试，多次失败请自行提取cookie")

@poe_login.got('code', prompt='请输入验证码')
async def __poe_login____(event: Event, state: T_State, infos: str = ArgStr("code")):
    global is_cookie_exists, global_playwright
    if infos in ["取消", "算了"]:
        await poe_login.finish("终止登陆")
    infos = infos.split(" ")
    if len(infos) != 1 or len(infos[0]) != 6:
        await poe_login.finish("你输入的验证码有误，请重新登陆")

    playwright = state["playwright"]
    page = state["page"]
    browser = state["browser"]
    context = state["context"]
    is_submitted_code = await submit_code(page, infos[0], cookie_path)
    if is_submitted_code:
        await browser.close()
        is_cookie_exists = True
        await poe_login.finish("登陆成功")
    else:
        await browser.close()
        await poe_login.finish("出错了，可以换个邮箱试试，多次失败请自行提取cookie")
        
######################################################
poe_addprompt = on_command(
    "poeaddprompt",
    aliases={
        "添加预设",
        "pap"
        },
    priority=4,
    block=False)

@poe_addprompt.handle()
async def __poe_addprompt__(event: Event):
    userid = str(event.user_id)
    if userid not in superusers:
        await poe_addprompt.finish("你不是管理员哦")

@poe_addprompt.got('name',prompt='请输入预设名称')
async def __poe_addprompt____(event: Event, state: T_State, infos: str = ArgStr("name")):
    global driver
    userid = str(event.user_id)
    if infos in ["取消", "算了"]:
        await poe_addprompt.finish("终止添加")
    infos = infos.split(" ")
    if len(infos) != 1:
        await poe_addprompt.reject("你输入的信息有误，请检查后重新输入")
    state["key"] = infos[0]
    
@poe_addprompt.got('prompt',prompt='请输入预设')
async def __poe_addprompt____(event: Event, state: T_State, infos: str = ArgStr("prompt")):
    global driver
    userid = str(event.user_id)
    if infos in ["取消", "算了"]:
        await poe_addprompt.finish("终止添加")
    infos = infos.split(" ")
    name = state["key"]
    prompt = infos[0]
    # # 将更新后的字典写回到JSON文件中
    prompts_dict[name] = prompt
    with open(prompt_path, 'w') as f:
        json.dump(prompts_dict, f)
    await poe_addprompt.finish("成功添加prompt")
    
######################################################
poe_removeprompt = on_command(
    "poeremoveprompt",
    aliases={
        "删除预设",
        "prp"
        },
    priority=4,
    block=False)

@poe_removeprompt.handle()
async def __poe_removeprompt__(event: Event):
    userid = str(event.user_id)
    if userid not in superusers:
        await poe_removeprompt.finish("你不是管理员哦")
    else:
        str_prompts = str()
        for key, _ in prompts_dict.items():
            str_prompts += f"{key}\n"
        await poe_removeprompt.send(f"当前预设有：\n{str_prompts}")

@poe_removeprompt.got('name',prompt='请输入预设名称')
async def __poe_removeprompt____(event: Event, infos: str = ArgStr("name")):
    global driver
    userid = str(event.user_id)
    if infos in ["取消", "算了"]:
        await poe_removeprompt.finish("终止添加")
    infos = infos.split(" ")
    if len(infos) != 1 or infos[0] not in prompts_dict:
        await poe_removeprompt.reject("你输入的信息有误，请检查后重新输入")
    del prompts_dict[infos[0]]
    with open(prompt_path, 'w') as f:   
        json.dump(prompts_dict, f)
    await poe_removeprompt.finish(f"成功删除预设{infos[0]}")
    
######################################################
poe_help = on_command(
    "poehelp",
    aliases={
        "poe帮助",
        "ph"
        },
    priority=4,
    block=False)
@poe_help.handle()
async def __poe_help__(bot: Bot,event: Event):
    msg = (
    "--注意所有功能都是用户独立的，每个用户只能操作自己的内容\n"
    "--所有分步操作都可以用 取消 或 算了来终止,并且支持错误重输\n"
    "--如果未创建机器人，对话命令将默认创建gpt3.5\n"
    "--可以直接回复机器人给你的回答来继续对话，无需命令\n"
    "--可以使用数字索引来使用建议回复\n\n"
    "--以下命令前面全部要加 / \n"
    "~对话:poetalk / ptalk / pt\n"
    "~清空历史对话:poedump / pdump / pd\n"
    "~创建机器人:poecreate / 创建bot / pc\n"
    "~删除机器人:poeremove / 删除bot / pr\n"
    "~切换机器人:poeswitch / 切换bot / ps\n\n"
    "************************\n"
    "--以下功能仅限poe管理员使用\n"
    "~登录:poelogin / plogin / pl\n"
    "~添加预设:poeaddprompt / 添加预设 / pap\n"
    "~删除预设:poeremoveprompt / 删除预设 / prp"
    )
    
    pic = await txt2img.draw(title="poe功能大全",text=msg)
    helpmsg = await poe_help.send(MessageSegment.image(pic))
    await asyncio.sleep(30)
    await bot.delete_msg(message_id=helpmsg['message_id'])
    await poe_help.finish()



import asyncio
import json

async def submit_email(page,email):
    for i in range(5):
        try:
            await page.goto("https://poe.com")
            # 点击 "Use email" 按钮
            use_email_button = await page.query_selector('button:has-text("Use email")')
            await use_email_button.click()

            # 填写 email 地址
            email_input = await page.wait_for_selector('input.EmailInput_emailInput__4v_bn')
            await email_input.fill(email)

            # 点击 "Go" 按钮
            go_button = await page.query_selector('button:has-text("Go")')
            await go_button.click()

            # 等待跳转并检查输入框是否存在
            await page.wait_for_selector('input.VerificationCodeInput_verificationCodeInput__YD3KV')
            return True
        except:
            await page.reload()
    return False


async def submit_code(page, code, path):
    retry_count = 0
    while retry_count < 3:
        try:
            # 填写验证码
            code_input = await page.wait_for_selector('input.VerificationCodeInput_verificationCodeInput__YD3KV')
            await code_input.fill('') # 清空输入框
            await code_input.fill(code)

            # 点击 "Log In" 按钮
            login_button = await page.query_selector('button:has-text("Log In")')
            await login_button.click()

            # 等待页面跳转，并检查页面是否有指定的输入框元素
            await page.wait_for_selector('textarea.ChatMessageInputView_textInput__Aervw')

            # 获取当前页面的 cookie，并保存 poe.com 域名下的 cookie 到本地文件中
            cookies = await page.context.cookies()
            poe_cookies = [cookie for cookie in cookies if cookie['domain'] == 'poe.com']
            with open(path, 'w') as f:
                json.dump(poe_cookies, f)

            return True
        except:
            # 如果页面上没有指定的输入框元素，就认为是失败了
            if not await page.query_selector('textarea.ChatMessageInputView_textInput__Aervw'):
                retry_count += 1
                # 等待一段时间后再次尝试输入验证码并点击 "Log In" 按钮
                await asyncio.sleep(1)
            else:
                raise
    return False
    
    


import asyncio
import base64
from io import BytesIO
import os
from pathlib import Path
from typing import Optional, Union
from PIL import Image, ImageDraw, ImageFont
from .poe_func import get_qr_img
from .config import Config
config = Config()
abs_path = os.path.abspath(__file__)
root = os.path.dirname(abs_path)
DATA_ROOT = root
DATA_PATH = DATA_ROOT + "/TXT2IMG"
FONT_PATH = DATA_PATH + "/font"
IMAGE_PATH = DATA_PATH + "/image"
FONT_FILE = FONT_PATH + "/sarasa-mono-sc-regular.ttf"
MI_BACKGROUND_FILE = IMAGE_PATH + "/mi_background.png"


templates = {
    "mi": {
        "font": str(FONT_FILE),
        "text": {
            "color": (125, 101, 89),
        },
        "title": {
            "color": (125, 101, 89),
        },
        "margin": 80,
        "background": {
            "type": "image",
            "image": str(MI_BACKGROUND_FILE),
        },
        "border": {
            "color": (220, 211, 196),
            "width": 2,
            "margin": 30,
        },
    },
    "simple": {
        "font": str(FONT_FILE),
        "text": {
            "color": (0, 0, 0),
        },
        "title": {
            "color": (0, 0, 0),
        },
        "margin": 50,
        "background": {
            "type": "color",
            "color": (255, 255, 255),
        },
    },
}


class Txt2Img:
    """Convert text to image"""

    font_family: str
    title_font_size: int
    text_font_size: int
    title_line_space: int
    text_line_space: int
    text_max_width: int
    fix_width: bool
    text_color: tuple
    title_color: tuple
    bg_color: tuple

    def __init__(self):
        self.raw_text = str()
        self.font_family = str(FONT_FILE)
        self.title_font_size = 45
        self.text_font_size = 30
        self.title_line_space = 30
        self.text_line_space = 15
        self.text_max_width = 1080
        self.fix_width = False
        self.text_color = (0, 0, 0, 255)
        self.title_color = (0, 0, 0, 255)
        self.bg_color = (255, 255, 255, 0)

    def set_font_family(self, font_family: str):
        """设置字体"""
        self.font_family = font_family

    def set_font_size(
            self,
            font_size: int,
            title_font_size: Optional[int] = None):
        """设置字体大小"""
        self.text_font_size = font_size
        self.text_line_space = font_size // 2
        if title_font_size is not None:
            self.title_font_size = title_font_size
        else:
            self.title_font_size = int(font_size * 1.5)
        self.title_line_space = font_size

    def set_font_color(
            self,
            text_color: tuple,
            title_color: Optional[tuple] = None):
        """设置字体颜色"""
        self.text_color = text_color
        if title_color is not None:
            self.title_color = title_color
        else:
            self.title_color = text_color

    def set_width(self, width: int):
        """设置图片宽度"""
        self.text_max_width = width
        self.fix_width = True

    async def word_wrap(self, text: str, font: ImageFont.FreeTypeFont) -> str:
        """异步自动换行"""
        temp_len = 0
        result = ""
        for ch in text:
            char_w = font.getsize(ch)[0]
            if ch == "\n":
                result += ch
                temp_len = 0
            elif char_w > 0:
                result += ch
                temp_len += char_w
                if temp_len > self.text_max_width - self.text_font_size:
                    temp_len = 0
                    result += "\n"
            await asyncio.sleep(0) # 让出控制权，避免阻塞事件循环
        result = result.rstrip()
        return result

    async def draw_img(self, title: str, text: str, template: Union[str, dict] = "mi") -> Image.Image:
        """异步绘制给定模板模板下的图片"""

        if isinstance(template, str):
            try:
                template = templates[template]  # type: ignore
            except KeyError:
                template = templates["mi"]  # type: ignore

        try:
            font_family = template["font"]  # type: ignore
            text_color = template["text"]["color"]  # type: ignore
            title_color = template["title"]["color"]  # type: ignore
            margin = int(template["margin"])  # type: ignore
            background = template["background"]  # type: ignore
        except KeyError:
            raise ValueError("Invalid template")

        if not Path(font_family).exists():
            raise ValueError("Invalid font")

        self.set_font_family(font_family)
        self.set_font_color(text_color, title_color)  # type: ignore
        text_img = await self.draw_text(title, text)
        # text_img.show()
        try:
            if background["type"] == "image":  # type: ignore
                out_img = Image.new(
                    "RGBA",
                    (text_img.width + 2 * margin, text_img.height + 2 * margin + (text_img.width + 2 * margin) //4),
                    (0, 0, 0, 0),
                )
                bg_img = await asyncio.to_thread(Image.open, background["image"])  # type: ignore
                out_img = await tile_image( bg_img, out_img)
            elif background["type"] == "color":  # type: ignore
                out_img = Image.new(
                    "RGBA",
                    (text_img.width + 2 * margin, text_img.height + 2 * margin),
                    background["color"])  # type: ignore
            else:
                raise ValueError("Invalid background type")
        except Exception:
            raise ValueError("Invalid template")

        out_img.paste(text_img, (margin, margin), text_img)
        # out_img.show()
        h=out_img.height
        w=out_img.width
        
        qrcode,url = await get_qr_img(self.raw_text)
        # 调整QRCode图像大小
        new_qr_img_width = out_img.width // 4
        new_qr_img_height=new_qr_img_width
        qr_img = qrcode.resize((new_qr_img_width, new_qr_img_height))
        out_img.paste(qr_img, ((w-new_qr_img_width)//2, h-margin-new_qr_img_height+20), None)
        # out_img.show()
        try:
            border = template["border"]  # type: ignore
            border_color = border["color"]  # type: ignore
            border_width = int(border["width"])  # type: ignore
            border_margin = int(border["margin"])  # type: ignore
            draw = ImageDraw.Draw(out_img)
            draw.rectangle(
                (
                    border_margin,
                    border_margin,
                    out_img.width - border_margin,
                    out_img.height - border_margin,
                ),
                outline=border_color,
                width=border_width,
            )
        except KeyError:
            pass
        except Exception:
            raise ValueError("Invalid template")
        # out_img.show()
        return out_img

    async def draw(self, title: str, text: str, template: Union[str, dict] = "mi") -> str:
        """异步绘制给定模板下指定标题与正文的图片并转换为base64"""
        self.raw_text = text
        out_img = await self.draw_img(title, text, template)
        return await img2b64(out_img)

    async def draw_text(self, title: str, text: str) -> Image.Image:
        """异步绘制标题与正文的图片"""
        title_font = ImageFont.truetype(self.font_family, self.title_font_size)
        text_font = ImageFont.truetype(self.font_family, self.text_font_size)

        if title == " ":
            title = ""

        if len(title.split("\n")) > 1:
            title = title.split("\n")[0]

        text = await self.word_wrap(text, text_font)

        lines = text.split("\n")
        text_rows = len(lines)

        title_width = title_font.getsize(title)[0]
        title_height = self.title_font_size + self.title_line_space

        if not self.fix_width:
            line_max_width = max([text_font.getsize(line)[0]
                                for line in lines])
            text_total_width = max(line_max_width, title_width)
        else:
            text_total_width = self.text_max_width

        text_height = self.text_font_size * (text_rows+1) + (text_rows - 1) * self.text_line_space
        if title:
            text_total_height = title_height + self.title_line_space + text_height
        else:
            text_total_height = text_height

        out_img = Image.new(
            mode="RGBA",
            size=(
                text_total_width,
                text_total_height),
            color=self.bg_color)
        draw = ImageDraw.Draw(out_img)

        if title:
            draw.text(
                ((text_total_width - title_width) // 2, 0),
                title,
                font=title_font,
                fill=self.text_color,
                spacing=self.title_line_space,
            )
            draw.text(
                (
                    0,
                    title_height + self.title_line_space,
                ),
                text,
                font=text_font,
                fill=self.text_color,
                spacing=self.text_line_space,
            )
        else:
            draw.text(
                (0, 0),
                text,
                font=text_font,
                fill=self.text_color,
                spacing=self.text_line_space,
            )
        return out_img


async def tile_image(small_image: Image.Image, big_image: Image.Image) -> Image.Image:
    """将小图片异步平铺到大图片上"""
    w, h = small_image.size
    queue = asyncio.Queue()

    for i in range(0, big_image.size[0], w):
        for j in range(0, big_image.size[1], h):
            await queue.put((i, j))

    async def paste_worker():
        while not queue.empty():
            i, j = await queue.get()
            big_image.paste(small_image, (i, j))

    workers = [asyncio.create_task(paste_worker()) for _ in range(8)]
    await asyncio.gather(*workers)
    return big_image

async def img2b64(img: Image.Image) -> str:
    """异步图片转 base64"""
    with BytesIO() as buffer:
        await asyncio.to_thread(img.save, buffer, format="PNG")
        base64_str = "base64://" + base64.b64encode(buffer.getvalue()).decode()
        return base64_str


import asyncio
import json
from playwright.async_api import async_playwright
from .config import Config
config = Config()
async def clear_bot(page, botname):
    try:
        await page.goto(f'https://poe.com/{botname}')
        
        # 等待元素出现
        element = await page.wait_for_selector('div.ChatMessageInputView_paintbrushWraper__DHMNW')
        await asyncio.sleep(1)
        await element.wait_for_selector('svg.ChatMessageInputView_paintbrushIcon__Turkx')
        svg_element = await page.query_selector(".ChatMessageInputView_paintbrushIcon__Turkx")

        # Get the bounding box of the SVG element
        bbox = await svg_element.bounding_box()
        x = bbox["x"] + 10   # Replace with your desired x coordinate
        y = bbox["y"] - 10   # Replace with your desired y coordinate
        await page.mouse.click(x=x,y=y)
        
        return True
    except:
        return False
    finally:
        await page.close()
        
async def poe_clear(cookie_path,botname):
    async with async_playwright() as p:
        server = config.server
        username = config.username
        passwd = config.passwd
        proxy_config = {}
        if server is not None:
            proxy_config["server"] = server
        if username is not None:
            proxy_config["username"] = username
        if passwd is not None:
            proxy_config["password"] = passwd

        if proxy_config:
            browser = await p.chromium.launch(proxy=proxy_config)
        else:
            browser = await p.chromium.launch()
        context = await browser.new_context()
        page = await context.new_page()
        with open(cookie_path, 'r') as f:
            cookies = json.load(f)
        await context.add_cookies(cookies=cookies)
        is_cleared = await clear_bot(page, botname)
        await context.close()
        await browser.close()
        return is_cleared
    


import asyncio
import json
import re
from playwright.async_api import async_playwright
from playwright.sync_api import Page
from .config import Config
config = Config()
async def send_message_async(page: Page, botname: str, input_str: str):
    # 定义重试次数和重试间隔时间
    retry_count = 5
    retry_interval = 0.5

    # 定义当前重试次数和报错次数
    current_retry = 0
    error_count = 0

    while current_retry < retry_count:

        try:
            # 打开目标网页
            await page.goto(f'https://poe.com/{botname}')
            text = "This bot has been deleted for violating our"
            content = await page.content()
            if text in content:
                return "banned"
            # 找到输入框元素
            input_box = await page.wait_for_selector('.ChatMessageInputView_textInput__Aervw',timeout=2000)

            # 将字符串发送到输入框中
            await input_box.fill(input_str)

            # 找到发送按钮元素并点击,容易点不到，多等一次
            await page.wait_for_selector('button.Button_primary__pIDjn',timeout=1000)
            send_button = await page.wait_for_selector('button.Button_primary__pIDjn',timeout=1000)
            # await asyncio.sleep(0.5)
            await send_button.click()
            # 发送成功后退出循环
            break
        except:
            # 如果出现超时异常，打印错误信息并稍等一段时间后重试
            error_count += 1
            if error_count >= 1:
                await asyncio.sleep(retry_interval)
            current_retry += 1
            continue
    if current_retry == retry_count:
        return False

async def get_message_async(page,botname, sleep):
    consecutive_errors = 0  # initialize a counter for consecutive errors
    while True:
        await asyncio.sleep(sleep)
        try:
            await page.goto(f'https://poe.com/{botname}')
            response = await page.content()
            text = "This bot has been deleted for violating our"
            if text in response:
                return "banned"
            match_text = re.search(r'<script id="__NEXT_DATA__" type="application/json">*(.*?)</script>', response, re.DOTALL)
            json_data_text = match_text.group(1)
            json_obj_text = json.loads(json_data_text)

            chat_list_raw = json_obj_text["props"]["pageProps"]["payload"]["chatOfBotDisplayName"]["messagesConnection"]["edges"]

            chat_list_raw = [a["node"] for a in chat_list_raw]
            chat_list_text = [a["text"] for a in chat_list_raw]
            if chat_list_text[-1]:
                match_suggest = re.search(r'<section class="ChatMessageSuggestedReplies_suggestedRepliesContainer__JgW12">*(.*?)</section>', response, re.DOTALL)
                string_list = re.findall(r'>\s*([^<>\n]+)\s*<', match_suggest.group(1))
                if len(string_list) == 4:
                    return chat_list_text, string_list
        except :
            consecutive_errors += 1
            print(consecutive_errors)
            if consecutive_errors >= 10:
                return False
        else:
            consecutive_errors = 0

async def poe_chat(cookie_path,botname,question):
    async with async_playwright() as p:
        server = config.server
        username = config.username
        passwd = config.passwd
        proxy_config = {}
        if server is not None:
            proxy_config["server"] = server
        if username is not None:
            proxy_config["username"] = username
        if passwd is not None:
            proxy_config["password"] = passwd

        if proxy_config:
            browser = await p.chromium.launch(proxy=proxy_config)
        else:
            browser = await p.chromium.launch()
        context = await browser.new_context()
        page = await context.new_page()
        with open(cookie_path, 'r') as f:
            cookies = json.load(f)
        await context.add_cookies(cookies=cookies)
        is_banned = await send_message_async(page, botname, question)
        if is_banned == "banned":
            return "banned"
        result = await get_message_async(page, botname, sleep=3)
        if isinstance(result, tuple):
            answers, suggests = result
            return answers[-1],suggests
        elif isinstance(result, str):
            is_banned = result
            return "banned"
        elif isinstance(result, bool):
            is_got = result
            return is_got
        else:
            raise ValueError("Unexpected return type from get_message_async")


import asyncio
from poe_func import get_qr_img
from txt2img import Txt2Img
txt2img = Txt2Img()

async def main():
    msg = '''是的，我还在。有什么问题我可以帮助您解答吗？

建议回复：
1: Tell me more.
2: 我有一个问题，你能帮我查找一下吗？
3: 我想了解一下你能提供哪些服务。
4: 你能告诉我如何使用你的功能吗？'''
    pic = await txt2img.draw(title=" ",text = msg)
    print(pic)
# asyncio.run(main())




import base64
from io import BytesIO
from typing import Union
import re, uuid, asyncio,string,random
import aiohttp
import qrcode
from nonebot import require
require("nonebot_plugin_guild_patch")
from nonebot_plugin_guild_patch import GuildMessageEvent
from nonebot.adapters.onebot.v11 import Message,MessageEvent,MessageSegment

def reply_out(event: MessageEvent, content: Union[MessageSegment,Message,str,bytes]) -> Message:
    """返回一个回复消息"""
    if isinstance(event, GuildMessageEvent):
        return Message(content)
    if type(content) == bytes:
        return MessageSegment.reply(event.message_id) + MessageSegment.image(content)
    if content[0:9] == 'base64://':
        return MessageSegment.reply(event.message_id) + MessageSegment.image(content)
    return MessageSegment.reply(event.message_id) + content

#生成一个由qq号和nickname共同决定的uuid作为真名，防止重名
def generate_uuid(s) ->str:
    # 将字符串转换为 UUID 对象
    uuid_object = uuid.uuid3(uuid.NAMESPACE_DNS, s)
    # 获取 UUID 对象的 bytes 值
    uuid_bytes = uuid_object.bytes
    # 将 bytes 值转换为字符串
    uuid_str = uuid_bytes.hex()[:14]
    return uuid_str

def generate_random_string(length=8) -> str:
    """生成指定长度的随机字符串"""
    letters = string.ascii_letters + string.digits
    return ''.join(random.choice(letters) for _ in range(length))

def is_email(email) -> bool:
    pattern = r'^[\w\.-]+@[\w\.-]+\.[a-zA-Z]{2,}$'
    return bool(re.match(pattern, email))

async def get_qr_img(text):
    """将 Markdown 文本保存到 Mozilla Pastebin，并获得 URL"""
    async with aiohttp.ClientSession() as session:
        payload = {'expires': '86400', 'format': 'url', 'lexer': '_markdown', 'content': text}
        try:
            async with session.post('https://pastebin.mozilla.org/api/',
                                    data=payload) as resp:
                resp.raise_for_status()
                url = await resp.text()
                url = url[0:-1]
        except Exception as e:
            url = f"上传失败：{str(e)}"
        image = qrcode.make(url)
        return image,url



import json
import os
import nonebot
from pathlib import Path    
def check_cookie(self):
    if os.path.exists(self.cookie_path):
        try:
            with open(self.cookie_path, 'r') as f:
                cookies = json.load(f)
            for cookie in cookies:
                if all(key in cookie for key in ('domain', 'name', 'path')) and 'value' in cookie:
                    if cookie['domain'] == 'poe.com' and cookie['name'] == 'p-b' and cookie['path'] == '/':
                        return True
        except:
            pass
    try:
        if not os.path.exists(self.cookie_path):
            with open(self.cookie_path, 'w') as f:
                f.write('{}')
                print('poe_cookie.json 创建成功')
        else:
            print('poe_cookie.json 已经存在')
        poe_ck = nonebot.get_driver().config.poe_cookie
        cookie_parms = [
        {
            "domain": "poe.com",
            "name": "p-b",
            "path": "/",
            "value": f"{poe_ck}"
        }
        ] 
        is_env_ck = True
    except:
        is_env_ck = False

    if is_env_ck:
        with open(self.cookie_path, 'w') as f:
            json.dump(cookie_parms, f)
        return True
    return False
class Config:
    def __init__(self):
        self.path_ = str(Path()) + "/data/poe_chat"
        self.user_path = str(self.path_ + r'/user_dict.json')
        self.prompt_path = str(self.path_ + r'/poe_prompt.json')
        self.cookie_path = str(self.path_ + r'/poe_cookie.json')
        self.pic_able = True
        self.server = None
        self.username = None
        self.passwd = None
        self.superusers = []
        self.user_dict = {}
        self.prompts_dict = {}
        self.is_cookie_exists = check_cookie(self)
        # 加载超级用户配置
        try:
            self.superusers = nonebot.get_driver().config.poe_superusers
        except:
            pass
        try:
            self.pic_able = nonebot.get_driver().config.poe_picable
        except:
            pass
        try:
            self.server = nonebot.get_driver().config.poe_server
        except:
            pass
        try:
            self.username = nonebot.get_driver().config.poe_username
        except:
            pass
        try:
            self.passwd = nonebot.get_driver().config.poe_passwd
        except:
            pass
        
        # 加载用户配置文件
        if not os.path.exists(self.user_path):
            # 获取目录路径
            dir_path = os.path.dirname(self.user_path)
            # 如果目录不存在，则创建目录
            if not os.path.exists(dir_path):
                os.makedirs(dir_path)
            with open(self.user_path, 'w') as f:
                f.write('{}')
                print('user_dict.json 创建成功')
        else:
            print('user_dict.json 已经存在')
            
        try:
            with open(self.user_path, 'r') as f:
                self.user_dict = json.load(f)
        except:
            pass
        
        # 检查并加载prompts配置文件
        if not os.path.exists(self.prompt_path):
            with open(self.prompt_path, 'w') as f:
                f.write('{}')
                print('poe_prompt.json 创建成功')
        else:
            print('poe_prompt.json 已经存在')
            
        try:
            with open(self.prompt_path, 'r') as f:
                self.prompts_dict = json.load(f)
        except:
            pass



import setuptools

with open("README.md", "r", encoding="utf-8") as f:
    long_description = f.read()

setuptools.setup(
    name="nonebot_poe_chat",  # 项目名称，保证它的唯一性，不要跟已存在的包名冲突即可
    version="1.0.0",  # 程序版本
    author="canxin",  # 项目作者
    author_email="1969730106@qq.com",  # 作者邮件
    description="nonebot_poe_chat",  # 项目的一句话描述
    long_description=long_description,  # 加长版描述
    long_description_content_type="text/markdown",  # 描述使用Markdown
    url="https://github.com/canxin121/nonebot_poe_chat",  # 项目地址
    packages=setuptools.find_packages(),  # 无需修改
    classifiers=[
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.8",
        "License :: OSI Approved :: GNU Affero General Public License v3",  # 开源协议
        "Operating System :: OS Independent",
    ],
    install_requires=[
        'nonebot2>=2.0.0rc3',
        'nonebot-adapter-onebot>=2.0.0b1',
        'nonebot_plugin_guild_patch>=0.2.3',
        'playwright>=1.32.1',
        'aiohttp>=3.8.4',
        'Pillow>=9.5.0',
        'qrcode>=7.4.2'

    ]
)
